pub mod core;
pub mod macros;
pub use core::parser::document as parse;

#[cfg(test)]
mod test {
    use std::{
        collections::HashMap,
        time::{Duration, SystemTime},
    };

    use crate::{
        core::element::{Element, Render},
        parse, read_param, text,
    };

    #[test]
    fn string_escape() {
        std::fs::write(
            "string_escape.html",
            parse(&std::fs::read_to_string("examples/string_escape.lisp").unwrap()).render_safe(),
        )
        .unwrap();

        assert_eq!(
            std::fs::read_to_string("string_escape.html").unwrap(),
            "\"Hello,\" world!Hello, \"w\"orld!<span x=\"y\">Hello</span>"
        )
    }

    #[test]
    fn plugins() {
        std::fs::write(
            "plugins.html",
            parse(&std::fs::read_to_string("examples/plugins.lisp").unwrap()).render(&mut {
                let mut h = HashMap::new();

                h.insert(
                    "say_hi".to_string(),
                    Box::new(|_| text!("Hello, world!")) as _,
                );

                h.insert(
                    "say_hi_with_params".to_string(),
                    Box::new(|e: Element| text!(read_param!(e, 0))) as _,
                );

                h.insert(
                    "say_hi_multiple".to_string(),
                    Box::new(|e: Element| {
                        let mut content: String = String::new();

                        for child in e.children {
                            let text = child.attrs.get("content").unwrap();
                            content += &format!("{text}, ");
                        }

                        text!(content)
                    }) as _,
                );

                h.insert(
                    "read_file".to_string(),
                    Box::new(|e: Element| {
                        text!(std::fs::read_to_string(read_param!(e, 0)).unwrap())
                    }) as _,
                );

                h
            }),
        )
        .unwrap();
    }

    #[test]
    fn boilerplate() {
        std::fs::write(
            "boilerplate.html",
            parse(&std::fs::read_to_string("examples/boilerplate.lisp").unwrap()).render_safe(),
        )
        .unwrap();
    }

    #[test]
    fn boilerplate_speed() {
        let start = SystemTime::now();

        std::fs::write(
            "boilerplate.html",
            parse(&std::fs::read_to_string("examples/boilerplate.lisp").unwrap()).render_safe(),
        )
        .unwrap();

        let duration = start.elapsed().unwrap();
        println!("took: {}μs", duration.as_micros());
        assert!(duration < Duration::from_micros(500))
    }
}
